<?php
/**
 * Extension of the DataTreeObject class for storing Page information in the
 * Categories driver. If you want to store specialized Page information, you
 * should extend this class instead of extending DataTreeObject directly.
 *
 * $Horde: giapeto/lib/Page.php,v 1.100 2007/01/02 12:47:38 jan Exp $
 *
 * Copyright 2002-2007 The Horde Project (http://www.horde.org/)
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Marko Djukic <marko@oblo.com>
 * @since   Giapeto 0.1
 * @package Giapeto
 */
class DataTreeObject_Page extends DataTreeObject {

    /**
     * The Giapeto object which this page came from - needed for updating data
     * in the backend to make changes stick, etc.
     *
     * @var Giapeto
     */
    var $_pageOb;

    /**
     * An array holding all apps loaded in a page.
     *
     * @var array
     */
    var $_loaded_apps = array('giapeto' => true);

    /**
     * The DataTreeObject_Page constructor. Just makes sure to call the parent
     * constructor so that the page's name is set properly.
     *
     * @param string $name The name of the page.
     */
    function DataTreeObject_Page($name)
    {
        parent::DataTreeObject($name);
    }

    /**
     * Associates a Giapeto object with this page.
     *
     * @param Giapeto $pageOb  The Giapeto object.
     */
    function setPageOb(&$pageOb)
    {
        $this->_pageOb = &$pageOb;
    }

    function getId()
    {
        return $this->_pageOb->getPageId($this);
    }

    /**
     * Get the short name for this page. Slightly different to the original
     * DataTreeObject function in that it displays the __root page as _("Home").
     *
     * @return string The page's short name.
     */
    function getShortName($page_path = '')
    {
        if (empty($page_path)) {
            $page_path = $this->name;
        }
        if ($page_path == '__root') {
            return _("Home");
        }
        $tmp = explode(':', $page_path);
        return array_pop($tmp);
    }

    /**
     * Parse the layout string to create the html table structure:
     *   x - replace with %s for use in vprintf
     *   / - close the tr and start a new one
     *   [ - start a new table
     *   ] - close the table
     *   | - close the td and open a new one
     *
     * @return string  with layout expanded to html and %s for vprintf
     */
    function getPageLayout()
    {
        if (isset($this->data['page']['layout'])) {
            return Giapeto::parsePageLayout($this->data['page']['layout']);
        }
        return false;
    }

    function getPageLayoutString()
    {
        if (isset($this->data['page']['layout'])) {
            return $this->data['page']['layout'];
        }
        return '';
    }

    function getPageAnnouncement()
    {
        if (isset($this->data['page']['announce'])) {
            return $this->data['page']['announce'];
        }
        return false;
    }

    function getPageModified($format = true)
    {
        if (isset($this->data['page']['modified'])) {
            if ($format) {
                return strftime('%d/%m/%y %H:%M', $this->data['page']['modified']);
            } else {
                return $this->data['page']['modified'];
            }
        }
        return null;
    }

    function isPageModified()
    {
        /* If this page has been updated today, we consider it
         * modified. */
        $today = strftime('%d/%m/%y', time());
        $modified = strftime('%d/%m/%y', $this->data['page']['modified']);
        return ($today == $modified);
    }

    function setPageAnnounce($announce, $message)
    {
        $this->data['page']['announce']['status'] = $announce;
        $this->data['page']['announce']['message'] = $message;
    }

    function getPageAnnounce()
    {
        if (isset($this->data['page']['announce'])) {
            return $this->data['page']['announce'];
        } else {
            return array('status' => $this->isPageModified(), 'message' => '');
        }
    }

    /**
     * Set the layout to the page object data array.
     *
     * @param string $layout  The layout of the page to store.
     */
    function setPageLayout($layout)
    {
        $this->data['page']['layout'] = $layout;
    }

    /**
     * Set the template to the page object data array.
     *
     * @param string $template  The template to set to the page object.
     */
    function setPageTemplate($template)
    {
        $this->data['page']['template'] = $template;
    }

    /**
     * Set the last modified time to the page object data array.
     */
    function setPageModified()
    {
        $this->data['page']['modified'] = time();
    }

    /**
     * Get the values of the page fields set in the page object.
     *
     * @param array $out  Array elements to which to add page data.
     *
     * @return array  Array with the page field values.
     */
    function getPageData(&$data)
    {
        if (isset($this->data['page']['layout'])) {
            $data['layout'] = $this->data['page']['layout'];
        }
        if (isset($this->data['page']['template'])) {
            $data['template'] = $this->data['page']['template'];
        }
        if (isset($this->data['page']['modified'])) {
            $data['modified'] = $this->data['page']['modified'];
        }
    }

    /**
     * Get total number of blocks in this page.
     *
     * @return integer  The number of blocks.
     */
    function getNumBlocks()
    {
        return substr_count($this->getPageLayoutString(), 'x');
    }

    /**
     * Sets any block data into the page category object, and stores
     * into the VFS any data that was updated.
     */
    function setBlockData($in)
    {
        $block_id = $in['id'];
        $type = $in['type'];

        /* Set up reference of page object data array to block var. */
        if (empty($this->data['blocks'][$block_id])) {
            $old_block = null;
        } else {
            $old_block = $this->data['blocks'][$block_id];
        }
        $new_block = array();

        /* Set the block title */
        $new_block['title'] = $in['title'];

        /* Figure out what the previous block was. */
        if (isset($old_block['text'])) {
            $old_type = 'text';
        } elseif (isset($old_block['richtext'])) {
            $old_type = 'richtext';
        } elseif (isset($old_block['file'])) {
            $old_type = 'file';
        } elseif (isset($old_block['image'])) {
            $old_type = 'image';
        } else {
            $old_type = '';
        }

        /* Set up VFS if needed. */
        if ($type == 'text' || $type == 'richtext' || $type == 'file' ||
            $type == 'image') {
            if (!empty($old_type) && isset($old_block[$old_type]['vfs_id'])) {
                /* Reuse the existing VFS id. */
                $new_block[$type]['vfs_id'] = $old_block[$old_type]['vfs_id'];
                $new_block[$type]['vfs_path'] = $old_block[$old_type]['vfs_path'];
            } else {
                /* No existing VFS id, create a new one. */
                $new_block[$type]['vfs_id']   = md5(mt_rand());
                $new_block[$type]['vfs_path'] = $this->_pageOb->getCategoryUrl($this->getName(), -1);
            }
        }

        /* Do any type-specific logic. */
        switch ($type) {
        case 'delete':
            /* Deleting block so delete VFS contents. */
            if (isset($old_block[$old_type]['vfs_id'])) {
                $delete_vfs = $GLOBALS['vfs']->deleteFile(GIAPETO_VFS_PATH . $old_block[$old_type]['vfs_path'], $old_block[$old_type]['vfs_id']);
            }
            unset($this->data['blocks'][$block_id]);
            return;

        case 'text':
            $new_block['link'] = $in['link'];
            $new_block['style'] = $in['style'];
            $new_block['style']['color'] = $in['color'];

            /* Write the text contents to VFS. */
            $write_vfs = $GLOBALS['vfs']->writeData(GIAPETO_VFS_PATH . $new_block['text']['vfs_path'], $new_block['text']['vfs_id'], $in['text'], true);
            if (is_a($write_vfs, 'PEAR_Error')) {
                Horde::logMessage(sprintf('Error saving text block to VFS, path \'%s\'. %s', GIAPETO_VFS_PATH . $new_block['text']['vfs_path'], $write_vfs->getMessage()), __FILE__, __LINE__, PEAR_LOG_ERR);
                return $write_vfs;
            }
            break;

        case 'richtext':
            /* Write the text contents to VFS. */
            $write_vfs = $GLOBALS['vfs']->writeData(GIAPETO_VFS_PATH . $new_block['richtext']['vfs_path'], $new_block['richtext']['vfs_id'], $in['richtext'], true);
            if (is_a($write_vfs, 'PEAR_Error')) {
                Horde::logMessage(sprintf('Error saving richtext block to VFS. %s', $write_vfs->getMessage()), __FILE__, __LINE__, PEAR_LOG_ERR);
                return $write_vfs;
            }
            break;

        case 'image':
            $new_block['link'] = $in['link'];
            $new_block['style']['text-align'] = $in['text-align'];
            $new_block['params']['name'] = $in['image']['name'];
            $new_block['params']['mime-type'] = $in['image']['type'];
            $new_block['params']['size'] = $in['image']['size'];

            if (!isset($in['image']['file'])) {
                /* No new file upload. */
                break;
            }

            /* Write the text contents to VFS. */
            $write_vfs = $GLOBALS['vfs']->write(GIAPETO_VFS_PATH . $new_block['image']['vfs_path'], $new_block['image']['vfs_id'], $in['image']['file'], true);
            if (is_a($write_vfs, 'PEAR_Error')) {
                Horde::logMessage(sprintf('Error saving image block to VFS. %s', $write_vfs->getMessage()), __FILE__, __LINE__, PEAR_LOG_ERR);
                return $write_vfs;
            }
            /* Write the original file if requested. */
            if ($in['image']['keep_orig']) {
                $new_block['image_zoom'] = $in['image']['keep_orig'];
                $write_vfs = $GLOBALS['vfs']->write(GIAPETO_VFS_PATH . $new_block['image']['vfs_path'], 'zoom_' . $new_block['image']['vfs_id'], $in['image']['orig_file'], true);
            }
            break;

        case 'file':
            $new_block['params']['name'] = $in['file']['name'];
            $new_block['params']['mime-type'] = $in['file']['type'];
            $new_block['params']['size'] = $in['file']['size'];

            /* Write the file contents to VFS. */
            $s = $GLOBALS['vfs']->write(GIAPETO_VFS_PATH . $new_block['file']['vfs_path'], $new_block['file']['vfs_id'], $in['file']['tmp_name'], true);
            break;

        case 'app':
            list($new_block['app'], $new_block['params']['type']) = explode(':', $in['block']['app']);
            /* Loop through options and save to the new block params element.
             * If options is an array as returned by mlenum get just the last
             * element. */
            foreach ($in['block']['options'] as $option_key => $option_val) {
                if (is_array($option_val)) {
                    $option_val = array_pop($option_val);
                }
                $new_block['params']['params'][$option_key] = $option_val;
            }
            break;
        }

        /* Set the new block. */
        $this->data['blocks'][$block_id] = $new_block;
    }

    /**
     * Swap the contents of two blocks.
     *
     * @return integer  The number of blocks.
     */
    function swapBlocks($block_1, $block_2)
    {
        /* Create the arrays if these blocks are not set. */
        if (!isset($this->data['blocks'][$block_1])) {
            $this->data['blocks'][$block_1] = array();
        }
        if (!isset($this->data['blocks'][$block_2])) {
            $this->data['blocks'][$block_2] = array();
        }

        /* Do the swap. */
        $block_tmp = $this->data['blocks'][$block_2];
        $this->data['blocks'][$block_2] = $this->data['blocks'][$block_1];
        $this->data['blocks'][$block_1] = $block_tmp;

        /* Clean up, don't bother storing empty blocks. */
        if (empty($this->data['blocks'][$block_1])) {
            unset($this->data['blocks'][$block_1]);
        }
        if (empty($this->data['blocks'][$block_2])) {
            unset($this->data['blocks'][$block_2]);
        }
    }

    /**
     * Return a nicely put together array of the page parts which
     * need to be displayed. Takes into account all the style
     * formatting which need to be applied to each section through
     * the use of the <div> tags.
     *
     * @param array $page_parts  An array of the various components
     *
     * @return array  with full section string in each element
     */
    function getBlocks()
    {
        global $giapeto_perms;

        if ($giapeto_perms->checkPermissions($this->getId(), 'page', PERMS_EDIT) &&
            !Util::getFormData('noedit')) {
            /* If editor make the block edit javascript available. */
            Horde::addScriptFile('blockedit.js');
            $editor = true;
        } else {
            $editor = false;
        }

        $blocks = array();
        $blocks_todo = $this->getNumBlocks();
        for ($b_id = 0; $b_id < $blocks_todo; $b_id++) {
            $blocks[$b_id] = '';
            if ($editor) {
                $blocks[$b_id] = $this->getBlockControls($b_id);
            }
            $blocks[$b_id] .= $this->getBlockContent($b_id);
        }

        return $blocks;
    }

    function getRawBlocks($block_id = null)
    {
        if (isset($this->data['blocks'])) {
            if (is_null($block_id)) {
                /* No specific block requested, return all. */
                return $this->data['blocks'];
            } elseif (isset($this->data['blocks'][$block_id])) {
                /* Specific block, and is set. */
                return $this->data['blocks'][$block_id];
            }
        }
        return array();
    }

    /**
     * Get the edit controls for the blocks - an HTML table above each
     * block which allows the block to be edited.
     *
     * @param  integer $block_id  The block id for which to display the
     *                            control.
     *
     * @return string             The HTML to display.
     */
    function getBlockControls($block_id)
    {
        global $registry;

        $url = Horde::applicationUrl('blockedit.php');
        $url = Util::addParameter($url, array('id' => $block_id, 'page_id' => $this->getId()));
        $label_edit = sprintf(_("Modify block %s"), $block_id + 1);
        $block_edit = Horde::link($url, $label_edit, null, 'blockedit', 'blockEdit(\'' . $url . '\'); return false;') . Horde::img('block_edit.png', $label_edit) . '</a>' . _("Block: ") . ($block_id+1);

        $url = Util::addParameter(Horde::applicationUrl('blockmove.php'),
                                  array('block_id' => $block_id,
                                        'page_id' => $this->getId()));
        $block_up = Horde::link(Util::addParameter($url, 'move', 'up'), _("Move up")) . Horde::img('nav/up.png', _("Move up"), '', $registry->getImageDir('horde')) . '</a>';
        $block_down = Horde::link(Util::addParameter($url, 'move', 'down'), _("Move down")) . Horde::img('nav/down.png', _("Move down"), '', $registry->getImageDir('horde')) . '</a>';

        return sprintf('<div class="control"><span style="float:right">%s%s</span>%s</div>', $block_up, $block_down, $block_edit);
    }

    /**
     * Get the block content and settings, for editing.
     *
     * @param integer $block  Id of the block to fetch.
     *
     * @return array  Block contents and settings array.
     */
    function getBlock($block_id)
    {
        $block = $this->getRawBlocks($block_id);
        if (empty($block)) {
            return array();
        }

        if (isset($block['text'])) {
            $block['type'] = 'text';
            $block['text'] = $GLOBALS['vfs']->read(GIAPETO_VFS_PATH . $block['text']['vfs_path'], $block['text']['vfs_id']);
        } elseif (isset($block['richtext'])) {
            $block['type'] = 'richtext';
            $block['richtext'] = $GLOBALS['vfs']->read(GIAPETO_VFS_PATH . $block['richtext']['vfs_path'], $block['richtext']['vfs_id']);
        } elseif (isset($block['image'])) {
            $block['type'] = 'image';
        } elseif (isset($block['file'])) {
            $block['type'] = 'file';
        } elseif (isset($block['app'])) {
            $block['type'] = 'app';
        } else {
            $block['type'] = 'text';
        }

        return $block;
    }

    /**
     * Get the block content, rendered for display.
     *
     * @param integer $block  Id of the block to fetch.
     *
     * @return mixed  Contents of the block.
     */
    function getBlockContent($block_id)
    {
        $block = $this->getRawBlocks($block_id);
        if (empty($block)) {
            return '';
        }

        /* Check if the title is set. If so, embed the block in a
         * table with a header. */
        if (isset($block['title']) && $block['title'] != '') {
            $header = '<table class="giapetoblock" cellspacing="0" width="100%"><tr><td class="giapetoheader">' . $block['title'] . '</td></tr><tr><td class="giapetocontent">';
            $footer = '</td></tr></table>';
        } else {
            $header = '';
            $footer = '';
        }

        if (isset($block['text'])) {
            $block = $header . $this->_blockHtml_text($block) . $footer;
        } elseif (isset($block['richtext'])) {
            $block = $header . $this->_blockHtml_richtext($block) . $footer;
        } elseif (isset($block['image'])) {
            $block = $header . $this->_blockHtml_image($block) . $footer;
        } elseif (isset($block['file'])) {
            $block = $header . $this->_blockHtml_file($block) . $footer;
        } elseif (isset($block['app'])) {
            $block = $header . $this->_blockHtml_app($block) . $footer;
        } else {
            $block = '';
        }

        /* Do some error checking. */
        if (is_a($block, 'PEAR_Error')) {
            Horde::logMessage(sprintf('Error displaying block, page_id: %s, error message: %s', $this->getId(), $block->getMessage()), __FILE__, __LINE__, PEAR_LOG_ERR);
            $block = sprintf(_("Block contents not available. %s"), $block->getMessage());
        }

        return $block;
    }

    function save()
    {
        /* Set the last modified timestamp. */
        $this->setPageModified();

        return $this->_pageOb->storePage($this);
    }

    /**
     * Returns an array of any apps loaded in this page.
     *
     * @return array
     */
    function getLoadedApps()
    {
        return array_keys($this->_loaded_apps);
    }

    function _blockHtml_text($block)
    {
        /* Get text from VFS. */
        $text = $GLOBALS['vfs']->read(GIAPETO_VFS_PATH . $block['text']['vfs_path'], $block['text']['vfs_id']);
        if (is_a($text, 'PEAR_Error')) {
            return $text;
        }

        /* Set up the style for the div tag. */
        $style = '';
        foreach ($block['style'] as $k => $v) {
            if ($k == 'font-size' || $k == 'font-style' ||
                $k == 'font-weight' || $k == 'text-align' ||
                $k == 'color') {
                $style .= $k . ':' . $v . ';';
            }
        }

        /* Set up any link. */
        $link = '';
        if (isset($block['link']) && !empty($block['link'])) {
            /* Check if internal page. */
            if (ctype_digit($block['link'])) {
                $block['link'] = Horde::applicationUrl($this->_pageOb->getIdUrl($block['link']), true);
            }
            $destination = sprintf(_("Go to: %s"), $block['link']);
            $link = ' ' . Horde::link($block['link'], $destination) . Horde::img('more.png', $destination, 'align="middle"') . '</a>';
        }

        /* Filter the text contents. */
        require_once 'Horde/Text/Filter.php';
        $text = nl2br($text);
        $text = Text_Filter::filter($text, array('linkurls', 'emails'),
                                    array(array(), array('always_mailto' => true, 'class' => 'pagelink')));

        return sprintf('<div style="%s">%s%s</div>', $style, $text, $link);
    }

    function _blockHtml_richtext($block)
    {
        /* Get text from VFS. */
        return $GLOBALS['vfs']->read(GIAPETO_VFS_PATH . $block['richtext']['vfs_path'], $block['richtext']['vfs_id']);
    }

    function _blockHtml_image($block)
    {
        $url = Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/images/view.php');
        $url = Util::addParameter($url, array('f' => $block['image']['vfs_id'], 's' => 'vfs', 'p' => GIAPETO_VFS_PATH . $block['image']['vfs_path'], 'n' => $block['params']['name']));

        $zoom = '';
        if (!empty($block['image_zoom'])) {
            Horde::addScriptFile('popup.js', 'horde', true);
            $zoom_url = Util::removeParameter($url, 'f');
            $zoom_url = Util::addParameter($zoom_url, 'f', 'zoom_' . $block['image']['vfs_id']);

            /* Get the dimensions of the zoomed image. */
            $img_data = $GLOBALS['vfs']->read(GIAPETO_VFS_PATH . $block['image']['vfs_path'], 'zoom_' . $block['image']['vfs_id']);
            if (is_a($img_data, 'PEAR_Error')) {
                return PEAR::raiseError(_("Block contents not available."));
            }

            require_once 'Horde/Image.php';
            $params = array('temp' => Horde::getTempDir());
            if (!empty($conf['image']['convert'])) {
                $zoom_image = &Horde_Image::singleton('im', $params);
            } else {
                $zoom_image = &Horde_Image::singleton('gd', $params);
            }
            $zoom_image->loadString('zoom_' . $block['image']['vfs_id'], $img_data);
            $img_dims = $zoom_image->getDimensions();

            /* Set popup dimensions according to zoom image size, but
             * never over 800x600. */
            $popup_x = ($img_dims['width'] + 20) > 800 ? 800 : $img_dims['width'] + 20;
            $popup_y = ($img_dims['height'] + 20) > 600 ? 600 : $img_dims['height'] + 20;
            $zoom = Horde::link($zoom_url, _("Zoom this image."), '', '', 'popup(this.href, ' . $popup_x . ', ' . $popup_y . '); return false;') . Horde::img('zoom.png', _("Zoom this image.")) . '</a>';
        }

        if (!empty($block['link'])) {
            if (ctype_digit($block['link'])) {
                /* Internal page, set a nice title and work out the url. */
                $title = sprintf(_("Go to: %s"), $this->_pageOb->getIdUrl($block['link'], -1));
                $block['link'] = Horde::applicationUrl($this->_pageOb->getIdUrl($block['link']), true);
            } elseif (substr($block['link'], 0, 7) == 'mailto:') {
                /* A mailto: link. */
                $title = sprintf(_("Mail to: %s"), substr($block['link'], 7));
            } else {
                $title = sprintf(_("Go to: %s"), $block['link']);

                /* Any other external url, set url to title and pass
                 * through Horde::externalUrl() for stripping out
                 * session information if necessary. Don't do this for
                 * links to the same server. */
                $block_url = @parse_url($block['link']);
                if (!empty($block_url['host'])) {
                    $block['link'] = Horde::externalUrl($block['link']);
                }
            }
            $image = Horde::link($block['link'], $title) . Horde::img($url, $title, '', '') . '</a>' . $zoom;
        } else {
            $image = Horde::img($url, '', '', '') . $zoom;
        }

        if (isset($block['style']['text-align'])) {
            return sprintf('<div style="text-align:%s;">%s</div>', $block['style']['text-align'], $image);
        } else {
            return $image;
        }
    }

    function _blockHtml_file($block)
    {
        global $mime_drivers, $mime_drivers_map;
        require_once 'Horde/MIME/Part.php';
        require_once 'Horde/MIME/Viewer.php';
        require_once 'Horde/MIME/Magic.php';
        require_once 'Horde/MIME/Contents.php';
        $result = Horde::loadConfiguration('mime_drivers.php', array('mime_drivers', 'mime_drivers_map'), 'horde');
        extract($result);
        $result = Horde::loadConfiguration('mime_drivers.php', array('mime_drivers', 'mime_drivers_map'), 'giapeto');
        require_once 'Horde/Array.php';
        $mime_drivers = Horde_Array::array_merge_recursive_overwrite($mime_drivers, $result['mime_drivers']);
        $mime_drivers_map = Horde_Array::array_merge_recursive_overwrite($mime_drivers_map, $result['mime_drivers_map']);

        $mime_part = &new MIME_Part($block['params']['mime-type'], '');

        $viewer = &MIME_Viewer::factory($mime_part);
        if (!$viewer) {
            return '';
        }

        /* Set up the view url. */
        $url = Horde::applicationUrl('view.php');
        $params = array('actionID'  => 'download',
                        'file_type' => $block['params']['mime-type'],
                        'vfs_id'    => $block['file']['vfs_id'],
                        'vfs_path'  => $block['file']['vfs_path'],
                        'file_name' => $block['params']['name']);
        $url = Util::addParameter($url, $params);

        /* Make sure we have the tooltips javascript. */
        Horde::addScriptFile('tooltip.js', 'horde', true);

        /* Set up the icon and the title. */
        if (!empty($block['params']['mime-type'])) {
            $mime_icon = MIME_Viewer::getIcon($block['params']['mime-type']);
        } else {
            $mime_icon = MIME_Viewer::getIcon(MIME_Magic::filenameToMIME($block['params']['name']));
        }
        $title = _("download") . ': ' . $block['params']['name'];
        $tooltip = $title . "\n" . sprintf(_("size: %s"), $block['params']['size']) . "\n" . sprintf(_("type: %s"), $block['params']['mime-type']);

        /* Return the link with a tooltip for further info on the download. */
        return Horde::linkTooltip($url, $title, '', '', '', $tooltip) . Horde::img($mime_icon, $title, 'align="middle"', '') . '&nbsp;' . $block['params']['name'] . '</a>&nbsp;&nbsp;';
    }

    function _blockHtml_app($block)
    {
        /* Register that this app has been loaded in the page. */
        $this->_loaded_apps[$block['app']] = true;

        require_once 'Horde/Block.php';
        require_once 'Horde/Block/Collection.php';
        $app = &Horde_Block_Collection::getBlock($block['app'],
                                                 $block['params']['type'],
                                                 isset($block['params']['params']) ? $block['params']['params'] : null);
        if (is_a($app, 'PEAR_Error')) {
            return $app;
        } elseif (is_null($app->_app)) {
            return PEAR::raiseError(_("Block contents not available."));
        }

        return $app->getContent();
    }

    function _toAttributes()
    {
        /* Do the page elements. */
        foreach ($this->data['page'] as $key => $value) {
            if ($key == 'announce') {
                $value = Horde_Serialize::serialize($value, SERIALIZE_UTF7_BASIC, NLS::getCharset());
            }
            $attributes[] = array('name'  => 'p',
                                  'key'   => $key,
                                  'value' => $value);
        }

        /* If no blocks to process return now. */
        if (!isset($this->data['blocks'])) {
            return $attributes;
        }

        /* Do the block elements. */
        foreach ($this->data['blocks'] as $id => $block) {
            foreach ($block as $key => $value) {
                if ($key == 'text' || $key == 'richtext' || $key == 'image' ||
                    $key == 'file' || $key == 'style' || $key == 'params') {
                    $value = Horde_Serialize::serialize($value, SERIALIZE_UTF7_BASIC, NLS::getCharset());
                }
                $attributes[] = array('name'  => 'b_' . $id,
                                      'key'   => $key,
                                      'value' => $value);
            }
        }

        return $attributes;
    }

    function _fromAttributes($attributes)
    {
        require_once 'Horde/Serialize.php';

        /* Initialise the data array. */
        $this->data = array();

        /* Initialise the two main sub-arrays, cuts down on isset()
         * checking elsewhere. */
        $this->data['page'] = array('modified' => null);
        $this->data['blocks'] = array();

        foreach ($attributes as $attr) {
            if ($attr['name'] == 'p') {
                if ($attr['key'] == 'announce') {
                    $attr['value'] = Horde_Serialize::unserialize($attr['value'], SERIALIZE_UTF7_BASIC, NLS::getCharset());
                }
                $this->data['page'][$attr['key']] = $attr['value'];
            } elseif (substr($attr['name'], 0, 2) == 'b_') {
                $key = substr($attr['name'], 2);
                if ($attr['key'] == 'text' || $attr['key'] == 'richtext' ||
                    $attr['key'] == 'image' || $attr['key'] == 'file' ||
                    $attr['key'] == 'style' || $attr['key'] == 'params') {
                    $attr['value'] = Horde_Serialize::unserialize($attr['value'], SERIALIZE_UTF7_BASIC, NLS::getCharset());
                }
                $this->data['blocks'][$key][$attr['key']] = $attr['value'];
            }
        }
    }

}
